#include "ColorButton.h"
 

ColorButton::ColorButton(BRect frame,BMessage* msg,rgb_color c,BString l) : BControl(frame,"colorbutton",NULL,msg,B_FOLLOW_NONE,B_WILL_DRAW)
{
	label = l;
	color = c;
	cw = NULL;
	bitmap= new BBitmap(Bounds(),B_RGB32,true);
	bitmap->Lock();
	bView = new BView(Bounds(),"view",B_FOLLOW_NONE, B_WILL_DRAW);
	bitmap->AddChild(bView);
	bitmap->Unlock();

	IsDown = false;
	IsOver = false;
	OkToInvoke = false;
}
void ColorButton::AttachedToWindow()
{
	BControl::AttachedToWindow();
	SetTarget(Parent());
}

void ColorButton::Draw(BRect r)
{
	Render();
}
void ColorButton::Render()
{
	bitmap->Lock();

	BRect r = bView->Bounds();	
	bView->SetDrawingMode(B_OP_COPY);

	bView->SetHighColor(0,255,0);//ui_color(B_PANEL_BACKGROUND_COLOR));
	bView->FillRect(r);
	r = BRect(0,0,14,14);
	bView->SetHighColor(color);
	bView->FillRect(r);
	
	if(IsDown)
		bView->SetHighColor(tint_color(color,B_DARKEN_2_TINT));
	else
		bView->SetHighColor(tint_color(color,B_LIGHTEN_2_TINT));
	
		bView->StrokeLine(BPoint(r.left,r.top),BPoint(r.left,r.bottom));
		bView->StrokeLine(BPoint(r.left,r.top),BPoint(r.right,r.top));
	if(IsDown)
		bView->SetHighColor(tint_color(color,B_DARKEN_1_TINT));
	else
		bView->SetHighColor(tint_color(color,B_LIGHTEN_1_TINT));


	if(IsDown)
		bView->SetHighColor(tint_color(color,B_LIGHTEN_2_TINT));
	else
		bView->SetHighColor(tint_color(color,B_DARKEN_2_TINT));

		bView->StrokeLine(BPoint(r.left,r.bottom),BPoint(r.right,r.bottom));
		bView->StrokeLine(BPoint(r.right,r.top),BPoint(r.right,r.bottom));
	if(IsDown)
		bView->SetHighColor(tint_color(color,B_LIGHTEN_1_TINT));
	else
		bView->SetHighColor(tint_color(color,B_DARKEN_1_TINT));
	
	BPoint point(r.RightBottom());
	bView->MovePenTo(BPoint(point.x+10,point.y));
	bView->SetHighColor(0,0,0);
	bView->DrawString(label.String());
	bView->Sync();
	DrawBitmapAsync(bitmap,Bounds(),Bounds());
	Flush();
	Sync();
	
	
	bitmap->Unlock();
}
void ColorButton::MouseDown(BPoint point)
{
	//SetMouseEventMask(B_POINTER_EVENTS);
	uint32 button;
	GetMouse(&point,&button);
	if(button == B_PRIMARY_MOUSE_BUTTON && IsEnabled())
	{
		IsDown = true;
		OkToInvoke = true;
		Render();
	}
}
void ColorButton::MouseUp(BPoint point)
{
	
	IsDown = false;
	Render();

	if(OkToInvoke)
	{
		Invoke();
		OkToInvoke = false;
	}
}
void ColorButton::MouseMoved(BPoint point,uint32 transit,const BMessage* msg)
{

	uint32 button;
	GetMouse(&point,&button);
	switch(transit)
	{
		case B_ENTERED_VIEW:
		{
			//always have !IsDown - default state
			/*if(button == B_PRIMARY_MOUSE_BUTTON)
			{
				OkToInvoke = true;
				IsDown = true;
				IsOver = true;
				Render();
			}*/
			//else
			//{
				IsOver = true;
				//OkToInvoke = true;
				//Render();
			//}			
			//Parent()->Parent()->MakeFocus(true);
		}break;
		case B_EXITED_VIEW:
		{
			IsOver = false;
			if(IsDown)
			{
				IsDown = false;
				Render();
			}
			
			OkToInvoke = false;
			IsDown = false;
			//Render();
			//Parent()->Parent()->MakeFocus(false);
		}break;
	}
	Parent()->MouseMoved(point,transit,msg);
	//if(transit == B_EXITED_VIEW
	/*else
	{
		IsDown = false;
		Invalidate();
	}
	*/

}
void ColorButton::MessageReceived(BMessage* msg)
{
	if(msg->WasDropped())
	{
		rgb_color*	Rgb;
		ssize_t		Size;
		// if there is RGB color
		if( (msg->FindData("RGBColor", B_RGB_COLOR_TYPE, (const void **)&Rgb, &Size) == B_OK))
		{
				color = *Rgb;
				Render();
				BControl::Invoke();
		}
	}
		
	switch(msg->what)
	{
		case COLOR_WINDOW_QUIT:
		{
			cw = NULL;
		}
		break;
		case COLOURPREFS_UPDATE:
		{
			rgb_color*	Rgb;
			ssize_t		Size;
			// if there is RGB color
			if( (msg->FindData("color", B_RGB_COLOR_TYPE, (const void **)&Rgb, &Size) == B_OK))
			{
				color = *Rgb;
				Render();
				BControl::Invoke();
			}
		
		
		}break;
		default:
		BControl::MessageReceived(msg);
		break;
	}
}
rgb_color ColorButton::Value()
{
	return color;
}
status_t ColorButton::Invoke(BMessage* message = NULL)
{	
	
	//start up chooser window
	
	if(cw != NULL)
	{
		cw->Activate(true);
		return B_OK;
	}

	BPoint p;
	uint32 buttons;
	//parent->
	GetMouse(&p,&buttons);
	//parent->
	ConvertToScreen(&p);
	p.x-=30;
	p.y-=30;
	float w=350,h=115;
	cw = new ColorWindow(BRect(p.x,p.y,p.x+w,p.y+h),new BMessenger(this),color);
	cw->Show();
	
	return B_OK;
}



